@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2021
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -24,7 +24,7 @@
 namespace polymake { namespace polytope {
 
 template <typename Scalar>
-perl::Object polarize(perl::Object p_in, perl::OptionSet options)
+BigObject polarize(BigObject p_in, OptionSet options)
 {
    bool
       RIF_property_read = false,
@@ -33,9 +33,9 @@ perl::Object polarize(perl::Object p_in, perl::OptionSet options)
    const bool no_coordinates = options["no_coordinates"];
    const bool isCone = !p_in.isa("Polytope");
 
-   perl::ObjectType t = p_in.type();
-   perl::Object p_out(t);
-   if ( isCone )
+   BigObjectType t = p_in.type();
+   BigObject p_out(t);
+   if (isCone)
       p_out.set_description() << "Cone dualized from " << p_in.name() << endl;
    else
       p_out.set_description() << "Polytope polarized from " << p_in.name() << endl;
@@ -47,7 +47,7 @@ perl::Object polarize(perl::Object p_in, perl::OptionSet options)
    }
 
    if (!no_coordinates) {
-      const int ambient_dim = p_in.give("CONE_AMBIENT_DIM");
+      const Int ambient_dim = p_in.give("CONE_AMBIENT_DIM");
 
       if ( !isCone ) {
          const bool is_centered = p_in.give("WEAKLY_CENTERED");
@@ -111,7 +111,7 @@ perl::Object polarize(perl::Object p_in, perl::OptionSet options)
 
       p_out.take("CONE_AMBIENT_DIM") << ambient_dim;
 
-      int ldim, cdim;
+      Int ldim, cdim;
       if (p_in.lookup("LINEALITY_DIM") >> ldim)
          p_out.take("CONE_DIM") << ambient_dim-ldim;
 
@@ -120,32 +120,24 @@ perl::Object polarize(perl::Object p_in, perl::OptionSet options)
    }
 
    // generate the combinatorial symmetry group on the facets
-   Array<Array<int>> gens;
-   if (p_in.lookup("GROUP.FACETS_ACTION.GENERATORS")>>gens){
-       perl::Object a("group::PermutationAction");
-       a.take("GENERATORS") << gens;
-
-       perl::Object g("group::Group");
-       g.set_description() << "symmetry group on vertices of polar of" << p_in.name() << endl;
-       g.set_name("fullCombinatorialGroupOnRays");
-
-       p_out.take("GROUP") << g;
-       if (isCone) {
-          p_out.take("GROUP.RAYS_ACTION") << a;
-       } else {
-          p_out.take("GROUP.VERTICES_ACTION") << a;
-       }
+   Array<Array<Int>> gens;
+   if (p_in.lookup("GROUP.FACETS_ACTION.GENERATORS") >> gens) {
+      BigObject a("group::PermutationAction", "GENERATORS", gens);
+      BigObject g("group::Group", "fullCombinatorialGroupOnRays");
+      g.set_description() << "symmetry group on vertices of polar of" << p_in.name() << endl;
+      p_out.take("GROUP") << g;
+      if (isCone) {
+         p_out.take("GROUP.RAYS_ACTION") << a;
+      } else {
+         p_out.take("GROUP.VERTICES_ACTION") << a;
+      }
    }
-   else if(p_in.lookup("GROUP.RAYS_ACTION.GENERATORS")>>gens){
-       perl::Object a("group::PermutationAction");
-       a.take("GENERATORS") << gens;
-
-       perl::Object g("group::Group");
-       g.set_description() << "symmetry group on facets of polar of" << p_in.name() << endl;
-       g.set_name("fullCombinatorialGroupOnFacets");
-       g.take("FACETS_ACTION") << a;
-
-       p_out.take("GROUP") << g;
+   else if (p_in.lookup("GROUP.RAYS_ACTION.GENERATORS") >> gens) {
+      BigObject a("group::PermutationAction", "GENERATORS", gens);
+      BigObject g("group::Group", "fullCombinatorialGroupOnFacets");
+      g.set_description() << "symmetry group on facets of polar of" << p_in.name() << endl;
+      g.take("FACETS_ACTION") << a;
+      p_out.take("GROUP") << g;
    }
 
    Array<std::string> labels;
@@ -159,7 +151,8 @@ perl::Object polarize(perl::Object p_in, perl::OptionSet options)
 }
 
 template <typename Scalar>
-perl::Object dual_cone(perl::Object p_in, perl::OptionSet options){
+BigObject dual_cone(BigObject p_in, OptionSet options)
+{
    return polarize<Scalar>(p_in, options);
 }
 
@@ -189,7 +182,7 @@ UserFunctionTemplate4perl("# @category Transformations"
                           "# | 1 -1 0"
                           "# | 1 0 1"
                           "# | 1 0 -1"
-                          "# @example [prefer cdd]"
+                          "# @example [prefer cdd] [require bundled:cdd]"
                           "# To dualize the cone over a hexagon and print its rays, do this:"
                           "# > $c = new Cone(INPUT_RAYS=>[[1,0,0],[1,1,0],[1,2,1],[1,2,2],[1,1,2],[1,0,1]]);"
                           "# > $cd = polarize($c);"
